﻿using System;

namespace Parade.Metadata.Asf
{
	/// <summary>
	/// タグ情報の型を定義します。
	/// 各値の実数は Windows Media Format の WMT_ATTR_DATATYPE に対応しています。
	/// </summary>
	public enum AsfTagDataType
	{
		/// <summary>
		/// 符号なし 32 ビット整数。
		/// </summary>
		UInt32 = 0,

		/// <summary>
		/// UTF-16LE 文字列。
		/// </summary>
		String = 1,

		/// <summary>
		/// バイト配列。
		/// </summary>
		Binary = 2,

		/// <summary>
		/// 真偽値。
		/// </summary>
		Bool = 3,
		
		/// <summary>
		/// 符号なし 64 ビット整数。
		/// </summary>
		UInt64 = 4,

		/// <summary>
		/// 符号なし 16 ビット整数。
		/// </summary>
		UInt16 = 5,
	
		/// <summary>
		/// GUID ( 128 ビット )。
		/// </summary>
		Guid = 6,
	}

	/// <summary>
	/// AsfTagDataType 列挙体の拡張クラスです。
	/// </summary>
	static class AsfTagDataTypeExtension
	{
		/// <summary>
		/// AsfTagDataType の値を TagDataType の値を取得します。
		/// </summary>
		/// <param name="type">AsfTagDataType の値。</param>
		/// <returns>TagDataType の値。</returns>
		public static TagDataType ToTagDataType( this AsfTagDataType type )
		{
			switch( type )
			{
			case AsfTagDataType.Bool:	return TagDataType.Bool;
			case AsfTagDataType.String:	return TagDataType.String;
			case AsfTagDataType.UInt16:	return TagDataType.UInt16;
			case AsfTagDataType.UInt32:	return TagDataType.UInt32;
			case AsfTagDataType.UInt64:	return TagDataType.UInt64;
			default:					return TagDataType.Binary;
			}
		}

		/// <summary>
		/// 指定された値の内容が適切な型を持っていることを調べます。
		/// </summary>
		/// <param name="type">AsfTagDataType の値。</param>
		/// <param name="value">値。</param>
		/// <returns>適切な型を持つ場合は true。それ以外は false。</returns>
		public static bool IsValidType( this AsfTagDataType type, object value )
		{
			switch( type )
			{
			case AsfTagDataType.Bool:	return ( value is bool   );
			case AsfTagDataType.Guid:	return ( value is Guid   );
			case AsfTagDataType.String:	return ( value is string );
			case AsfTagDataType.UInt16:	return ( value is ushort );
			case AsfTagDataType.UInt32:	return ( value is uint   );
			case AsfTagDataType.UInt64:	return ( value is ulong  );
			default:					return ( value is byte[] );
			}
		}
	}
}
